@@ -80,10 +80,13 @@ static QString translatedCategory(const QString &category)
     return translatedCategories[index];
 }
 
+static EffectStatus effectStatus(bool enabled)
+{
+    return enabled ? EffectStatus::Enabled : EffectStatus::Disabled;
+}
+
 EffectModel::EffectModel(QObject *parent)
     : QAbstractItemModel(parent) {
-
-    loadEffects();
 }
 
 QHash< int, QByteArray > EffectModel::roleNames() const
@@ -162,7 +165,7 @@ QVariant EffectModel::data(const QModelIndex &index, int role) const
         case ServiceNameRole:
             return m_effectsList.at(index.row()).serviceName;
         case EffectStatusRole:
-            return m_effectsList.at(index.row()).effectStatus;
+            return (int)m_effectsList.at(index.row()).effectStatus;
         case VideoRole:
             return m_effectsList.at(index.row()).video;
         case SupportedRole:
@@ -186,11 +189,15 @@ bool EffectModel::setData(const QModelIndex& index, const QVariant& value, int r
         return QAbstractItemModel::setData(index, value, role);
 
     if (role == EffectModel::EffectStatusRole) {
+        // note: whenever the StatusRole is modified (even to the same value) the entry
+        // gets marked as changed and will get saved to the config file. This means the
+        // config file could get polluted
         EffectData &data = m_effectsList[index.row()];
-        data.effectStatus = value.toBool();
+        data.effectStatus = EffectStatus(value.toInt());
+        data.changed = true;
         emit dataChanged(index, index);
 
-        if (data.effectStatus && !data.exclusiveGroup.isEmpty()) {
+        if (data.effectStatus == EffectStatus::Enabled && !data.exclusiveGroup.isEmpty()) {
             // need to disable all other exclusive effects in the same category
             for (int i = 0; i < m_effectsList.size(); ++i) {
                 if (i == index.row()) {
@@ -198,17 +205,13 @@ bool EffectModel::setData(const QModelIndex& index, const QVariant& value, int r
                 }
                 EffectData &otherData = m_effectsList[i];
                 if (otherData.exclusiveGroup == data.exclusiveGroup) {
-                    otherData.effectStatus = false;
+                    otherData.effectStatus = EffectStatus::Disabled;
+                    otherData.changed = true;
                     emit dataChanged(this->index(i, 0), this->index(i, 0));
                 }
             }
         }
 
-        return true;
-    } else if (role == EffectModel::WindowManagementRole) {
-        bool enabled = value.toBool();
-        handleWindowManagement(index.row(), enabled);
-        emit dataChanged(index, index);
         return true;
     }
 
@@ -236,7 +239,15 @@ void EffectModel::loadEffects()
         effect.category = translatedCategory(data.category);
         effect.serviceName = data.name;
         effect.enabledByDefault = data.enabled;
-        effect.effectStatus = kwinConfig.readEntry(effect.serviceName + "Enabled", effect.enabledByDefault);
+        effect.enabledByDefaultFunction = (data.enabledFunction != nullptr);
+        const QString enabledKey = QStringLiteral("%1Enabled").arg(effect.serviceName);
+        if (kwinConfig.hasKey(enabledKey)) {
+            effect.effectStatus = effectStatus(kwinConfig.readEntry(effect.serviceName + "Enabled", effect.enabledByDefault));
+        } else if (data.enabledFunction != nullptr) {
+            effect.effectStatus = EffectStatus::EnabledUndeterminded;
+        } else {
+            effect.effectStatus = effectStatus(effect.enabledByDefault);
+        }
         effect.video = data.video;
         effect.supported = true;
         effect.exclusiveGroup = data.exclusiveCategory;
@@ -264,8 +275,9 @@ void EffectModel::loadEffects()
         effect.version = plugin.version();
         effect.category = translatedCategory(plugin.category());
         effect.serviceName = plugin.pluginName();
-        effect.effectStatus = kwinConfig.readEntry(effect.serviceName + "Enabled", plugin.isPluginEnabledByDefault());
+        effect.effectStatus = effectStatus(kwinConfig.readEntry(effect.serviceName + "Enabled", plugin.isPluginEnabledByDefault()));
         effect.enabledByDefault = plugin.isPluginEnabledByDefault();
+        effect.enabledByDefaultFunction = false;
         effect.video = service->property(QStringLiteral("X-KWin-Video-Url"), QVariant::Url).toUrl();
         effect.supported = true;
         effect.exclusiveGroup = service->property(QStringLiteral("X-KWin-Exclusive-Category"), QVariant::String).toString();
@@ -335,13 +347,6 @@ void EffectModel::loadEffects()
     endResetModel();
 }
 
-void EffectModel::handleWindowManagement(int row, bool enabled)
-{
-    //Make sure that our row is valid
-    if (m_effectsList.size() > 0 && row <= m_effectsList.size())
-        m_effectsList[row].effectStatus = enabled;
-}
-
 int EffectModel::findRowByServiceName(const QString &serviceName)
 {
     for (int it = 0; it < m_effectsList.size(); it++) {
@@ -359,7 +364,7 @@ void EffectModel::syncEffectsToKWin()
                                              QDBusConnection::sessionBus());
     for (int it = 0; it < m_effectsList.size(); it++) {
         if (m_effectsList.at(it).effectStatus != m_effectsChanged.at(it).effectStatus) {
-            if (m_effectsList.at(it).effectStatus) {
+            if (m_effectsList.at(it).effectStatus != EffectStatus::Disabled) {
                 interface.loadEffect(m_effectsList.at(it).serviceName);
             } else {
                 interface.unloadEffect(m_effectsList.at(it).serviceName);
@@ -370,24 +375,31 @@ void EffectModel::syncEffectsToKWin()
     m_effectsChanged = m_effectsList;
 }
 
-void EffectModel::updateEffectStatus(const QModelIndex &rowIndex, bool effectState)
+void EffectModel::updateEffectStatus(const QModelIndex &rowIndex, EffectStatus effectState)
 {
-    setData(rowIndex, effectState, EffectModel::EffectStatusRole);
+    setData(rowIndex, (int)effectState, EffectModel::EffectStatusRole);
 }
 
 void EffectModel::syncConfig()
 {
     KConfigGroup kwinConfig(KSharedConfig::openConfig("kwinrc"), "Plugins");
 
-    for (auto it = m_effectsList.constBegin(); it != m_effectsList.constEnd(); it++) {
-        const EffectData &effect = *(it);
+    for (auto it = m_effectsList.begin(); it != m_effectsList.end(); it++) {
+        EffectData &effect = *(it);
+        if (!effect.changed) {
+            continue;
+        }
+        effect.changed = false;
 
         const QString key = effect.serviceName + QStringLiteral("Enabled");
-
-        if (effect.effectStatus != effect.enabledByDefault) {
-            kwinConfig.writeEntry(key, effect.effectStatus);
-        } else {
+        const bool shouldEnable = (effect.effectStatus != EffectStatus::Disabled);
+        const bool restoreToDefault = effect.enabledByDefaultFunction
+                                      ? effect.effectStatus == EffectStatus::EnabledUndeterminded
+                                      : shouldEnable == effect.enabledByDefault;
+        if (restoreToDefault) {
             kwinConfig.deleteEntry(key);
+        } else {
+            kwinConfig.writeEntry(key, shouldEnable);
         }
     }
 
@@ -399,8 +411,10 @@ void EffectModel::defaults()
 {
     for (int i = 0; i < m_effectsList.count(); ++i) {
         const auto &effect = m_effectsList.at(i);
-        if (effect.effectStatus != effect.enabledByDefault) {
-            updateEffectStatus(index(i, 0), effect.enabledByDefault);
+        if (effect.enabledByDefaultFunction && effect.effectStatus != EffectStatus::EnabledUndeterminded) {
+            updateEffectStatus(index(i, 0), EffectStatus::EnabledUndeterminded);
+        } else if ((bool)effect.effectStatus != effect.enabledByDefault) {
+            updateEffectStatus(index(i, 0), effect.enabledByDefault ? EffectStatus::Enabled : EffectStatus::Disabled);
         }
     }
 }
@@ -477,11 +491,11 @@ bool EffectFilterModel::filterAcceptsRow(int source_row, const QModelIndex &sour
     return false;
 }
 
-void EffectFilterModel::updateEffectStatus(int rowIndex, bool effectState)
+void EffectFilterModel::updateEffectStatus(int rowIndex, int effectState)
 {
     const QModelIndex sourceIndex = mapToSource(index(rowIndex, 0));
 
-    m_effectModel->updateEffectStatus(sourceIndex, effectState);
+    m_effectModel->updateEffectStatus(sourceIndex, EffectStatus(effectState));
 }
 
 void EffectFilterModel::syncConfig()
@@ -514,6 +528,7 @@ void EffectView::init(ViewType type)
 {
     KDeclarative::KDeclarative kdeclarative;
     kdeclarative.setDeclarativeEngine(engine());
+    kdeclarative.setTranslationDomain(QStringLiteral(TRANSLATION_DOMAIN));
     kdeclarative.setupBindings();
     QString path;
     switch (type) {
