@@ -998,7 +998,7 @@ void RuleBook::edit(AbstractClient* c, bool whole_app)
     p->setArguments(args);
     p->setProcessEnvironment(kwinApp()->processStartupEnvironment());
     p->setProgram(QStringLiteral(KWIN_RULES_DIALOG_BIN));
-    connect(p, static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished), this, &QProcess::deleteLater);
+    connect(p, static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished), p, &QProcess::deleteLater);
     connect(p, static_cast<void (QProcess::*)(QProcess::ProcessError)>(&QProcess::error), this,
         [p] (QProcess::ProcessError e) {
             if (e == QProcess::FailedToStart) {
