@@ -25,6 +25,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "client.h"
 #include "screens.h"
 #include "workspace.h"
+#if HAVE_WAYLAND
+#include "shell_client.h"
+#include "wayland_server.h"
+#endif
 
 namespace KWin {
 namespace ScriptingClientModel {
@@ -37,39 +41,44 @@ static quint32 nextId() {
 ClientLevel::ClientLevel(ClientModel *model, AbstractLevel *parent)
     : AbstractLevel(model, parent)
 {
-    connect(Workspace::self(), SIGNAL(clientAdded(KWin::Client*)), SLOT(clientAdded(KWin::Client*)));
-    connect(Workspace::self(), SIGNAL(clientRemoved(KWin::Client*)), SLOT(clientRemoved(KWin::Client*)));
+    connect(Workspace::self(), &Workspace::clientAdded, this, &ClientLevel::clientAdded);
+    connect(Workspace::self(), &Workspace::clientRemoved, this, &ClientLevel::clientRemoved);
     connect(model, SIGNAL(exclusionsChanged()), SLOT(reInit()));
+#if HAVE_WAYLAND
+    if (waylandServer()) {
+        connect(waylandServer(), &WaylandServer::shellClientAdded, this, &ClientLevel::clientAdded);
+    }
+#endif
 }
 
 ClientLevel::~ClientLevel()
 {
 }
 
-void ClientLevel::clientAdded(Client *client)
+void ClientLevel::clientAdded(AbstractClient *client)
 {
     setupClientConnections(client);
     checkClient(client);
 }
 
-void ClientLevel::clientRemoved(Client *client)
+void ClientLevel::clientRemoved(AbstractClient *client)
 {
     removeClient(client);
 }
 
-void ClientLevel::setupClientConnections(Client *client)
+void ClientLevel::setupClientConnections(AbstractClient *client)
 {
-    connect(client, SIGNAL(desktopChanged()), SLOT(checkClient()));
-    connect(client, SIGNAL(screenChanged()), SLOT(checkClient()));
-    connect(client, SIGNAL(activitiesChanged(KWin::Toplevel*)), SLOT(checkClient()));
+    auto check = [this, client] {
+        checkClient(client);
+    };
+    connect(client, &AbstractClient::desktopChanged, this, check);
+    connect(client, &AbstractClient::screenChanged, this, check);
+    connect(client, &AbstractClient::activitiesChanged, this, check);
+    connect(client, &AbstractClient::windowHidden, this, check);
+    connect(client, &AbstractClient::windowShown, this, check);
 }
 
-void ClientLevel::checkClient()
-{
-    checkClient(static_cast<Client*>(sender()));
-}
-
-void ClientLevel::checkClient(Client *client)
+void ClientLevel::checkClient(AbstractClient *client)
 {
     const bool shouldInclude = !exclude(client) && shouldAdd(client);
     const bool contains = containsClient(client);
@@ -81,7 +90,7 @@ void ClientLevel::checkClient(Client *client)
     }
 }
 
-bool ClientLevel::exclude(Client *client) const
+bool ClientLevel::exclude(AbstractClient *client) const
 {
     ClientModel::Exclusions exclusions = model()->exclusions();
     if (exclusions == ClientModel::NoExclusion) {
@@ -150,7 +159,7 @@ bool ClientLevel::exclude(Client *client) const
     return false;
 }
 
-bool ClientLevel::shouldAdd(Client *client) const
+bool ClientLevel::shouldAdd(AbstractClient *client) const
 {
     if (restrictions() == ClientModel::NoRestriction) {
         return true;
@@ -173,7 +182,7 @@ bool ClientLevel::shouldAdd(Client *client) const
     return true;
 }
 
-void ClientLevel::addClient(Client *client)
+void ClientLevel::addClient(AbstractClient *client)
 {
     if (containsClient(client)) {
         return;
@@ -183,10 +192,10 @@ void ClientLevel::addClient(Client *client)
     emit endInsert();
 }
 
-void ClientLevel::removeClient(Client *client)
+void ClientLevel::removeClient(AbstractClient *client)
 {
     int index = 0;
-    QMap<quint32, Client*>::iterator it = m_clients.begin();
+    auto it = m_clients.begin();
     for (; it != m_clients.end(); ++it, ++index) {
         if (it.value() == client) {
             break;
@@ -218,6 +227,14 @@ void ClientLevel::reInit()
     for (ClientList::const_iterator it = clients.begin(); it != clients.end(); ++it) {
         checkClient((*it));
     }
+#if HAVE_WAYLAND
+    if (waylandServer()) {
+        const auto &clients = waylandServer()->clients();
+        for (auto *c : clients) {
+            checkClient(c);
+        }
+    }
+#endif
 }
 
 quint32 ClientLevel::idForRow(int row) const
@@ -225,7 +242,7 @@ quint32 ClientLevel::idForRow(int row) const
     if (row >= m_clients.size()) {
         return 0;
     }
-    QMap<quint32, Client*>::const_iterator it = m_clients.constBegin();
+    auto it = m_clients.constBegin();
     for (int i=0; i<row; ++i) {
         ++it;
     }
@@ -240,7 +257,7 @@ bool ClientLevel::containsId(quint32 id) const
 int ClientLevel::rowForId(quint32 id) const
 {
     int row = 0;
-    for (QMap<quint32, Client*>::const_iterator it = m_clients.constBegin();
+    for (auto it = m_clients.constBegin();
             it != m_clients.constEnd();
             ++it, ++row) {
         if (it.key() == id) {
@@ -250,18 +267,18 @@ int ClientLevel::rowForId(quint32 id) const
     return -1;
 }
 
-Client *ClientLevel::clientForId(quint32 child) const
+AbstractClient *ClientLevel::clientForId(quint32 child) const
 {
-    QMap<quint32, Client*>::const_iterator it = m_clients.constFind(child);
+    auto it = m_clients.constFind(child);
     if (it == m_clients.constEnd()) {
         return nullptr;
     }
     return it.value();
 }
 
-bool ClientLevel::containsClient(Client *client) const
+bool ClientLevel::containsClient(AbstractClient *client) const
 {
-    for (QMap<quint32, Client*>::const_iterator it = m_clients.constBegin();
+    for (auto it = m_clients.constBegin();
             it != m_clients.constEnd();
             ++it) {
         if (it.value() == client) {
@@ -313,14 +330,16 @@ AbstractLevel *AbstractLevel::create(const QList< ClientModel::LevelRestriction
     switch (restriction) {
     case ClientModel::ActivityRestriction: {
 #ifdef KWIN_BUILD_ACTIVITIES
-        const QStringList &activities = Activities::self()->all();
-        for (QStringList::const_iterator it = activities.begin(); it != activities.end(); ++it) {
-            AbstractLevel *childLevel = create(childRestrictions, childrenRestrictions, model, currentLevel);
-            if (!childLevel) {
-                continue;
+        if (Activities::self()) {
+            const QStringList &activities = Activities::self()->all();
+            for (QStringList::const_iterator it = activities.begin(); it != activities.end(); ++it) {
+                AbstractLevel *childLevel = create(childRestrictions, childrenRestrictions, model, currentLevel);
+                if (!childLevel) {
+                    continue;
+                }
+                childLevel->setActivity(*it);
+                currentLevel->addChild(childLevel);
             }
-            childLevel->setActivity(*it);
-            currentLevel->addChild(childLevel);
         }
         break;
 #else
@@ -404,9 +423,10 @@ ForkLevel::ForkLevel(const QList<ClientModel::LevelRestriction> &childRestrictio
     connect(VirtualDesktopManager::self(), SIGNAL(countChanged(uint,uint)), SLOT(desktopCountChanged(uint,uint)));
     connect(screens(), SIGNAL(countChanged(int,int)), SLOT(screenCountChanged(int,int)));
 #ifdef KWIN_BUILD_ACTIVITIES
-    Activities *activities = Activities::self();
-    connect(activities, SIGNAL(added(QString)), SLOT(activityAdded(QString)));
-    connect(activities, SIGNAL(removed(QString)), SLOT(activityRemoved(QString)));
+    if (Activities *activities = Activities::self()) {
+        connect(activities, SIGNAL(added(QString)), SLOT(activityAdded(QString)));
+        connect(activities, SIGNAL(removed(QString)), SLOT(activityRemoved(QString)));
+    }
 #endif
 }
 
@@ -625,10 +645,10 @@ int ForkLevel::rowForId(quint32 child) const
     return -1;
 }
 
-Client *ForkLevel::clientForId(quint32 child) const
+AbstractClient *ForkLevel::clientForId(quint32 child) const
 {
     for (QList<AbstractLevel*>::const_iterator it = m_children.constBegin(); it != m_children.constEnd(); ++it) {
-        if (Client *client = (*it)->clientForId(child)) {
+        if (AbstractClient *client = (*it)->clientForId(child)) {
             return client;
         }
     }
@@ -696,7 +716,7 @@ QVariant ClientModel::data(const QModelIndex &index, int role) const
         }
     }
     if (role == Qt::DisplayRole || role == ClientRole) {
-        if (Client *client = m_root->clientForId(index.internalId())) {
+        if (AbstractClient *client = m_root->clientForId(index.internalId())) {
             return qVariantFromValue(client);
         }
     }
